<?php

require_once('includes/srv_tarifa.php');
require_once('includes/srv_servicio.php');
require_once('includes/srv_sucursal.php');
require_once('includes/srv_envio.php');
require_once('includes/srv_normalizacion.php');
require_once('includes/utilidades.php');

function correoschile_shipping_init() {
    if (!class_exists('CorreosChile_Shipping_Method')) {
        class CorreosChile_Shipping_Method extends WC_Shipping_Method {
            public function __construct() {
                $this->id = 'correoschile_shipping_method';
                $this->method_title = __('CorreosChile', 'correoschile_shipping_method');
                $this->method_description = __('Método de envío de CorreosChile', 'correoschile_shipping_method');
                $this->availability = 'including';
                $this->countries = array('CL');
                $this->init();
                $this->enabled = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';
                $this->title = isset( $this->settings['title'] ) ? $this->settings['title'] : __('CorreosChile', 'correoschile_shipping_method');
            }

            function init() {
                $this->init_form_fields();
                $this->init_settings();
                add_action('woocommerce_update_options_shipping_' . $this->id, array($this,'process_admin_options'));
            }

            function init_form_fields() { 
                $this->form_fields = array(
                    'enabled' => array(
                    'title' => __('Activo', 'correoschile_shipping_method' ),
                    'type' => 'checkbox',
                    'description' => __('Activar el método de envío.', 'correoschile_shipping_method' ),
                    'default' => 'yes'
                    ),
                    'title' => array(
                        'title' => __('Nombre', 'correoschile_shipping_method' ),
                        'type' => 'text',
                        'description' => __('Este es el nombre que aparecerá para el usuario.', 'correoschile_shipping_method' ),
                        'default' => __('CorreosChile', 'correoschile_shipping_method')
                    )
                );
            }

            public function calculate_shipping($package = array()) {
                //wc_add_notice(json_encode($package), 'notice');

                $mostrarMensajes = get_option('correoschile_mostrarMensajes', 'N');
                $subTotal = intval($package["cart_subtotal"]);
                $obtenerComunaDesde = get_option("correoschile_obtenerComunaDesde", "city");
                $comunaDestinatario = null;
                switch ($obtenerComunaDesde) { 
                    case "city":
                        $city = $package["destination"]["city"]; 
                        $comunaDestinatario = srv_comuna::ObtenerComunaPorNombre($city);
                        break;
                    case "state": 
                        $states = WC()->countries->get_states("CL");
                        $state = $package["destination"]["state"];
                        if (is_array($states)) {
                            $state = $states[$state];
                        }
                        $comunaDestinatario = srv_comuna::ObtenerComunaPorNombre($state);
                        break;
                }
                $nombreComunaRemitente = get_option("correoschile_comuna");
                if ($comunaDestinatario === null) {
                    if ($mostrarMensajes === 'S') {
                        wc_add_notice("<strong>" . $this->title . "</strong> informa:<br>No se encontró la comuna del destinatario", "notice");
                    }
                    return;
                }
                if ($comunaDestinatario["estaDisponible"] === "N") {
                    if ($mostrarMensajes === 'S') {
                        wc_add_notice("<strong>" . $this->title . "</strong> informa:<br>La comuna del destinatario no se encuentra disponible", "notice");
                    }
                    return;
                }

                $pesoTotal = 0;
                $volumenTotal = 0;
                foreach ($package["contents"] as $item_id => $values) { 
                    $producto = $values["data"];
                    $peso  = is_numeric($producto->get_weight()) ? wc_get_weight($producto->get_weight(), "kg") : 0;
                    $ancho = is_numeric($producto->get_width())  ? $producto->get_width()  : 0;
                    $largo = is_numeric($producto->get_length()) ? $producto->get_length() : 0;
                    $alto  = is_numeric($producto->get_height()) ? $producto->get_height() : 0;
                    $volumen = wc_get_dimension($ancho * $largo * $alto, "cm");
                    if($volumen === 0) {
                        $usarVolumenPorDefecto = get_option('correoschile_usarVolumenPorDefecto', 'N');
                        if($usarVolumenPorDefecto === 'S') {
                            $volumenPorDefecto = get_option('correoschile_volumenPorDefecto', '0');
                            $volumen = is_numeric($volumenPorDefecto) ? floatval($volumenPorDefecto) : 0;
                            if ($volumen < 0) { 
                                $volumen = 0; 
                            }
                        }
                    }
                    if($peso === 0)  {
                        $usarPesoPorDefecto = get_option('correoschile_usarPesoPorDefecto', 'N');
                        if($usarPesoPorDefecto === 'S') {
                            $pesoPorDefecto = get_option('correoschile_pesoPorDefecto', '0');
                            $peso = is_numeric($pesoPorDefecto) ? floatval($pesoPorDefecto) : 0;
                            if ($peso < 0) { 
                                $peso = 0; 
                            }
                        }
                    }
                    if($volumen > 0) {
                        $volumen = $volumen / 1000000;
                    }
                    $pesoTotal += $peso * $values["quantity"];
                    $volumenTotal += $volumen * $values["quantity"];
                }

                $usarPesoMaximo = get_option('correoschile_usarPesoMaximo', 'N');
                $pesoMaximo = get_option('correoschile_pesoMaximo', '0');
                $pesoMaximo = is_numeric($pesoMaximo) ? floatval($pesoMaximo) : 0;
                $pesoMaximo = $pesoMaximo < 0 ? 0 : $pesoMaximo;
                if($usarPesoMaximo === 'S' && $pesoTotal > $pesoMaximo) {
                    return;
                }
                try {
                    $tarifas = srv_tarifa::ObtenerTarifas($nombreComunaRemitente, 
                                                          $comunaDestinatario["nombre"], 
                                                          $pesoTotal, 
                                                          $volumenTotal);
                } 
                catch(Exception $exception) {
                    if ($mostrarMensajes === 'S') {
                        wc_add_notice("<strong>" . $this->title . "</strong> informa:<br>" . $exception->getMessage(), "error");
                    }
                    return;
                }
                try { 
                    $serviciosDisponibles = srv_servicio::ObtenerServiciosDisponibles();
                }
                catch(Exception $exception) {
                    if ($mostrarMensajes === 'S') {
                        wc_add_notice("<strong>" . $this->title . "</strong> informa:<br>" . $exception->getMessage(), "error");
                    }
                    return;
                }

                $sucursalesCargadas = false;
                foreach($serviciosDisponibles as $servicioDisponible) {
                    foreach($tarifas as $tarifa) {
                        if($tarifa->CodigoServicio !== $servicioDisponible["codigoServicio"]) {
                            continue;
                        }
                        $totalCalculado = $this->calcularTotal($comunaDestinatario, $tarifa->TotalTasacion->Total, $subTotal);
                        if ($servicioDisponible["esSucursal"] === "S") {
                            if($sucursalesCargadas === false) {
                                try {
                                    $sucursales = srv_sucursal::ObtenerSucursalesPorComuna($comunaDestinatario["idComuna"]);
                                }
                                catch(Exception $exception) {
                                    if ($mostrarMensajes === 'S') {
                                        wc_add_notice("<strong>" . $this->title . "</strong> informa:<br>" . $exception->getMessage(), "error");
                                    }
                                    return;
                                }
                                $sucursalesCargadas = true;
                            }
                            if(is_array($sucursales) === true) {
                                foreach($sucursales as $sucursal) {
                                    if (isset($sucursal->IataOficina) === true) {
                                        $rate = array(
                                            "id" =>  $this->id . "_S_" . $tarifa->CodigoServicio . "_" . $sucursal->IataOficina,
                                            "label" => $this->title . " - " . $servicioDisponible["descripcion"] . " - " . $sucursal->DescripcionOficina . ($totalCalculado === 0 ? ' (Gratis)' : ''),
                                            "cost" => $totalCalculado
                                        );
                                        $this->add_rate($rate);
                                    }
                                }
                            }
                            else {
                                if (isset($sucursales->IataOficina) === true) {
                                    $rate = array(
                                        "id" =>  $this->id . "_S_" . $tarifa->CodigoServicio . "_" . $sucursales->IataOficina,
                                        "label" => $this->title . " - " . $servicioDisponible["descripcion"] . " - " . $sucursales->DescripcionOficina . ($totalCalculado === 0 ? ' (Gratis)' : ''),
                                        "cost" => $totalCalculado
                                    );
                                    $this->add_rate($rate);
                                }
                            }
                        } 
                        else {
                            $rate = array(
                                "id" =>  $this->id . "_D_" . $tarifa->CodigoServicio,
                                "label" => $this->title . " - " . $servicioDisponible["descripcion"] . ($totalCalculado === 0 ? ' (Gratis)' : ''),
                                "cost" => $totalCalculado
                            );
                            $this->add_rate($rate);
                        }
                    }
                }
            }

            function calcularTotal($comunaDestinatario, $totalTarifa, $subTotal) {
                $totalCalculado = $totalTarifa;
                if ($comunaDestinatario['esGratis'] === "S") {
                    $esGratisDesde = intval($comunaDestinatario['esGratisDesde']);
                    if($esGratisDesde <= 0) {
                        $totalCalculado = 0;
                    } else {
                        if ($esGratisDesde <= $subTotal) {
                            $totalCalculado = 0;
                        }
                    }
                } else {
                    $precioFijoValor = $comunaDestinatario['precioFijoValor'];
                    if($comunaDestinatario['precioFijo'] === 'S' && is_numeric($precioFijoValor)) {
                        $totalCalculado = intval($precioFijoValor);
                    } else {
                        $totalCalculado = $this->marginar_costo_envio($comunaDestinatario, $totalTarifa);
                    }
                }
                return $totalCalculado;
            }

            function marginar_costo_envio($comunaDestinatario, $totalTarifa) {
                if($totalTarifa === 0) {
                    return $totalTarifa;
                }
                $valor = $comunaDestinatario['marginarValor'];
                if(is_numeric($valor) === false){
                    $valor = 0;
                }
                switch($comunaDestinatario['marginar']){
                    case 'P': 
                        $totalTarifa += ($valor * $totalTarifa)/100;
                        break;
                    case 'V': 
                        $totalTarifa = $totalTarifa + intval($valor);
                        break;                                                    
                }
                if($totalTarifa < 0) {
                    $totalTarifa = 0;
                }
                return round($totalTarifa, 0);
            }
        }
        add_action('woocommerce_after_checkout_validation', 'correoschile_validar_checkout', 10, 2);
        add_action('woocommerce_thankyou', 'correoschile_pedido_recibido', 10, 1);
    }
}

function correoschile_add_shipping_method($methods) {
    $methods['correoschile_shipping_method'] = 'CorreosChile_Shipping_Method';
    return $methods;
}

function correoschile_validar_checkout($data, $errors) {
    //$errors->add( 'validation', json_encode($data) );
    
    $shipping_methods = $data['shipping_method'];
    if (is_array($shipping_methods)) {
        $shipping_method = array_filter ($shipping_methods, function($k, $v) {
            return substr($k, 0, strlen('correoschile_shipping_method')) === 'correoschile_shipping_method';
        }, ARRAY_FILTER_USE_BOTH);
        if (count($shipping_method) > 0) {
            $normalizar = get_option('correoschile_normalizador', 'N');
            if ($normalizar === 'S') {
                $pais = $data['shipping_country'];
                if($pais !== 'CL') {
                    $errors->add( 'validation', 'El país seleccionado solo puede ser Chile.' );
                } else {
                    $obtenerComunaDesde = get_option('correoschile_obtenerComunaDesde', 'city');
                    $state = $data['shipping_state'];
                    $city = $data['shipping_city'];
                    $address1 = $data['shipping_address_1'];
                    $address2 = $data['shipping_address_2'];
                    switch ($obtenerComunaDesde) { 
                        case "state": 
                            $states = WC()->countries->get_states("CL");
                            if (is_array($states)) {
                                $state = $states[$state];
                            }
                            $comunaDestinatario = srv_comuna::ObtenerComunaPorNombre($state);
                        break;                    
                        case "city":
                            $comunaDestinatario = srv_comuna::ObtenerComunaPorNombre($city);
                        break;
                    }
                    if ($comunaDestinatario !== null) {
                        $nombreComuna = $comunaDestinatario["nombre"];
                        $normalizada = srv_normalizacion::IntentarNormalizarDireccion($address1, $nombreComuna);
                        if ($normalizada === false) {
                            $errors->add( 'validation', 'No se ha podido determinar el código postal de su dirección, por favor ingrese su dirección correctamente.' );
                        }
                    }
                }
            }
        }
    }
} 

function correoschile_pedido_recibido($order_id) {
    $order = wc_get_order($order_id);
    $data = $order->get_data();

    if ($order->has_shipping_method('correoschile_shipping_method')) {
        $chosen_methods = WC()->session->get('chosen_shipping_methods');
        $explodeChosenMethod = explode('_', $chosen_methods[0]);

        $sucursalDestino = "";
        $normalizada = "N";
        $codigoServicio = $explodeChosenMethod[4];
        if ($explodeChosenMethod[3] === "S") {
            $sucursal = srv_sucursal::ObtenerSucursal($explodeChosenMethod[5]);
            if ($sucursal !== null) {
                $sucursalDestino = $sucursal->DescripcionOficina;
            }
        }

        $obtenerComunaDesde = get_option("correoschile_obtenerComunaDesde", "city");
        $comunaDestinatario = null;
        $nombreComuna = "";
        $city = $data["shipping"]["city"];
        $state = $data["shipping"]["state"];
        switch ($obtenerComunaDesde) { 
            case "city":
                $comunaDestinatario = srv_comuna::ObtenerComunaPorNombre($city);
            break;
            case "state": 
                $states = WC()->countries->get_states("CL");
                if (is_array($states)) {
                    $state = $states[$state];
                }
                $comunaDestinatario = srv_comuna::ObtenerComunaPorNombre($state);
            break;
        }
        if ($comunaDestinatario !== null) {
            $nombreComuna = $comunaDestinatario["nombre"];
            $normalizada = srv_normalizacion::IntentarNormalizarDireccion($data["shipping"]["address_1"], 
                                                                          $nombreComuna);
            $normalizada = $normalizada === true ? "S" : "N";
        }

        $kilos = 0;
        $volumen = 0;
        $items = $order->get_items();
        $productFactory = new WC_Product_Factory();
        $tipoMercancia = '';
        foreach ($items as $item) {
            $producto = $productFactory->get_product($item['product_id']);
            $sku = $producto->get_sku();
            if(utilidades::IsNullOrEmptyString($sku) === false){
                if ($tipoMercancia !== '') {
                    $tipoMercancia .= ', ';
                }
                $tipoMercancia .= $sku .' x ' . strval($item['quantity']);
            }
            $peso = is_numeric($producto->get_weight()) ? wc_get_weight($producto->get_weight(), 'kg') : 0;
            $width = is_numeric($producto->get_width()) ? $producto->get_width() : 0;
            $length = is_numeric($producto->get_length()) ? $producto->get_length() : 0;
            $height = is_numeric($producto->get_height()) ? $producto->get_height() : 0;
            $kilos = $kilos + $peso * $item['quantity'];
            $volumen = $volumen + (wc_get_dimension($width * $length * $height, 'cm') / 1000000) * $item['quantity'];
        }
        $volumen = round($volumen, 3);

        $envio = srv_envio::ObtenerEnvio($order_id);
        if ($envio === null) {
            srv_envio::AgregarEnvio($order_id, $codigoServicio, $kilos, $volumen, $sucursalDestino, $normalizada, $tipoMercancia);
        }

        /*
        echo "<br>obtenerComunaDesde: " . $obtenerComunaDesde . 
             "<br>order_id: " . $order_id . 
             "<br>codigoServicio: " . $codigoServicio . 
             "<br>kilos: " . $kilos .
             "<br>volumen: " . $volumen .
             "<br>sucursalDestino: " . $sucursalDestino .
             "<br>normalizada: " . $normalizada .
             "<br>state: " . $state .
             "<br>city: " . $city .
             "<br>address_1: " . $data["shipping"]["address_1"] .
             "<br>nombreComuna: " . $nombreComuna .
             "<br>tipoMercancia: " . $tipoMercancia;
        */        
    }
}