<?php

require_once 'srv_envio.php';

class srv_anulacion {
    public static function AjaxAnularEtiqueta() {
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $tieneEnvio = false;
            try {
                $order_id = filter_input(INPUT_POST, 'order_id');
                if(!isset($order_id)) {
                    throw new Exception('No se proporciono la variable "order_id"');
                }
                $order = wc_get_order(intval($order_id)); 
                if(is_bool($order)) {
                    throw new Exception('No se encontró la información del pedido ' . $order_id);
                }
                $envio = srv_envio::ObtenerEnvio($order_id);
                if(is_null($envio)) {
                    throw new Exception('No se encontró la información del envío ' . $order_id);
                }
                $tieneEnvio = true;
                if(is_null($envio->numeroReferencia)) {
                    throw new Exception('No es posible anular un pedido sin etiqueta');
                }
                $anular_data = srv_anulacion::AnularEtiqueta($envio->numeroEnvio, $envio->codigoAdmision);
                srv_envio::ActualizarEnvio_Anular($order_id);
                srv_anulacion::LimpiarDirectorio($envio->numeroEnvio);
            
                header('http/1.1 200 OK');
                header('Content-Type: application/json; charset=UTF8');
                echo json_encode(array('message' => $anular_data));
                exit();
            } catch(Exception $e) {
                if($tieneEnvio === true) {
                    srv_envio::ActualizarEnvio_InfoError($order_id, $e->getMessage());
                }
                header('http/1.1 500 Internal Server Error');
                header('Content-Type: application/json; charset=UTF8');
                die(json_encode(array('message' => $e->getMessage())));
            }
        }
    }

    public static function AjaxEliminarEnvio() {

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $tieneEnvio = false;
            try {
                $order_id = filter_input(INPUT_POST, 'order_id');
                if(!isset($order_id)) {
                    throw new Exception('No se proporciono la variable "order_id"');
                }
                $order = wc_get_order(intval($order_id));
                if(is_bool($order)) {
                    throw new Exception('No se encontró la información del pedido ' . $order_id);
                }
                $envio = srv_envio::ObtenerEnvio($order_id);
                if(is_null($envio)) {
                    throw new Exception('No se encontró la información del envío ' . $order_id);
                }
                $tieneEnvio = true;
                if(!is_null($envio->numeroEnvio)) {
                    try {
                        srv_anulacion::AnularEtiqueta($envio->numeroEnvio, $envio->codigoAdmision);
                        srv_envio::ActualizarEnvio_Anular($order_id);
                        limpiarDirectorio($envio->numeroEnvio);
                    } catch(Exception $e) {
                        srv_envio::EliminarEnvio($order_id);
                        srv_anulacion::LimpiarDirectorio($envio->numeroEnvio);
                        //throw new Exception('Error al consultar servicio de anulación');
                    }
                }else{
                    srv_envio::EliminarEnvio($order_id);
                }
                header('http/1.1 200 OK');
                header('Content-Type: application/json; charset=UTF8');
                echo json_encode(array('message' => 'Pedido ' . $order_id . ' eliminado correctamente'));
                exit();
            } catch(Exception $e) {
                if ($tieneEnvio === true) {
                    srv_envio::ActualizarEnvio_InfoError($order_id, $e->getMessage());
                }
                header('http/1.1 500 Internal Server Error');
                header('Content-Type: application/json; charset=UTF8');
                die(json_encode(array('message' => $e->getMessage())));
            }
        }
    }

    public static function AnularEtiqueta($numeroEnvio, $codigoAdmision) {
        $wsdl = trim(get_option('correoschile_ws_anulacion', ''));
        if($wsdl !== '' && substr_compare(strtolower($wsdl), '?wsdl', -strlen('?wsdl')) < 0) {
            $wsdl .= '?wsdl';
        }
        try {
            $client = new SoapClient($wsdl, array('trace' => 1, 'exception' => 0));
            $result = $client->__soapCall('anularEnvio', array(
                'anularEnvio' => array(
                    'usuario' => trim(get_option('correoschile_usuario_api')),
                    'contrasena' => trim(get_option('correoschile_contrasena_api')),
                    'numeroEnvio' => $numeroEnvio,
                    'codigoAdmision' => $codigoAdmision,
                    'clienteRemitente' => get_option('correoschile_codigoSAP','')
                )), null, null);
            return $result->anularEnvioResult;
        }
        catch (SoapFault $e) {
            throw new Exception($e->getMessage(), 0, $e);
        }
        catch (Exception $e) {
            throw new Exception('Ocurrio un error intentando realizar la anulación del envío.', 0, $e);
        }
    }

    public static function LimpiarDirectorio($nroEnvio){
        $upload_dir   = wp_upload_dir();
        $upload_dirname = $upload_dir['basedir'];
        $upload_dirname .= '/correoschile_etiquetas';
        $etiqueta = $upload_dirname . '/' . $nroEnvio . '.pdf';
      
        if(file_exists($etiqueta)) {
            $eliminado = @unlink($etiqueta);
            if (!$eliminado) {
                 throw new Exception("No se ha podido eliminar etiqueta ".$nroEnvio);
            }
        }    
    }
}