<?php

require_once 'srv_servicio.php';
require_once 'srv_envio.php';
require_once 'srv_normalizacion.php';
require_once 'utilidades.php';

class srv_etiqueta {
    public static function AjaxGenerarEtiqueta() {
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $tieneEnvio = false;
            try {
                $order_id = filter_input(INPUT_POST, 'order_id');
                if(!isset($order_id)) {
                    throw new Exception('No se proporciono la variable "order_id"');
                }
                $order = wc_get_order(intval($order_id));
                if(is_bool($order)) {
                    throw new Exception('No se encontró la información del pedido ' . $order_id);
                }
                $envio = srv_envio::ObtenerEnvio($order_id);
                if(is_null($envio)) {
                    throw new Exception('No se encontró la información del envío ' . $order_id);
                }
                $tieneEnvio = true;

                $comprobarEstados = get_option("correoschile_comprobar_estado", "");
                switch ($comprobarEstados) {
                    case "processing":
                        $estado = $order->get_status();
                        if($estado !== 'processing') {
                            throw new Exception('Solo pueden generarse etiquetas para pedidos en proceso');        
                        }
                        break;
                    case "completed":
                        $estado = $order->get_status();
                        if($estado !== 'completed') {
                            throw new Exception('Solo pueden generarse etiquetas para pedidos completados');        
                        }
                        break;
                    case "processing,completed":
                        $estado = $order->get_status();
                        if($estado !== 'processing' || $estado !== 'completed') {
                            throw new Exception('Solo pueden generarse etiquetas para pedidos en proceso o completados');        
                        }                        
                        break;
                }

                if(utilidades::IsNullOrEmptyString($envio->numeroReferencia) === false) {
                    header('http/1.1 500 Internal Server Error');
                    header('Content-Type: application/json; charset=UTF8');
                    die(json_encode(array('message' => 'Ya existe una etiqueta para este envío.')));
                }
                $nombreDestinatario = $order->get_shipping_first_name() . ' ' . 
                                      $order->get_shipping_last_name();

                $direccionDestinatario = '';
                $servicio = srv_servicio::ObtenerServicio($envio->codigoServicio);
                if($servicio["esSucursal"] === 'N') {
                    $direccionDestinatario = $order->get_shipping_address_1();
                    $direccionDestinatario2 = $order->get_shipping_address_2();
                    if(!utilidades::IsNullOrEmptyString($direccionDestinatario2)){
                        $direccionDestinatario .= '. '. $direccionDestinatario2;
                    }
                }
                if($servicio["esSucursal"] === 'S') {
                    $direccionDestinatario = $envio->sucursalDestino;
                }

                $obtenerComunaDesde = get_option("correoschile_obtenerComunaDesde", "city");
                switch ($obtenerComunaDesde) {
                    case 'city': $comunaDestinatario = $order->get_shipping_city(); break;
                    case 'state': $comunaDestinatario = $order->get_shipping_state(); break;                        
                }
                $telefonoDestinatario = $order->get_billing_phone();
                $codigoServicio = $envio->codigoServicio;
                $numeroTotalPiezas = $envio->numeroTotalPiezas;
                $kilos = $envio->kilos;
                $volumen = $envio->volumen;
                
                $numeroReferencia = $order_id; //substr($order->get_order_key(),9);
                $prefijo = get_option("correoschile_prefijo", "");
                if($prefijo !== '') {
                    $numeroReferencia = $prefijo . $numeroReferencia;
                }

                $observaciones = $order->get_customer_note();
                $observaciones2 = $envio->observaciones2;
                $emailDestinatario = $order->get_billing_email();
                $formatoEtiqueta = get_option('correoschile_formato_etiqueta','PDF');
                $valorDeclarado = $order->get_total();
                $pagoSeguro = $envio->pagoSeguro;
                $tipoMercancia = $envio->tipoMercancia;
                        
                srv_etiqueta::ValidarGeneracionEtiqueta(
                    $order_id, $nombreDestinatario, $direccionDestinatario, 
                    $comunaDestinatario, $telefonoDestinatario, $codigoServicio,
                    $numeroTotalPiezas, $kilos, $volumen, $numeroReferencia, 
                    $observaciones, $observaciones2, $emailDestinatario, 
                    $formatoEtiqueta, $valorDeclarado, $servicio["esSucursal"],
                    $pagoSeguro, $tipoMercancia);
                
                $numeroEnvio = srv_etiqueta::GenerarEtiqueta(
                    $order_id, $nombreDestinatario, $direccionDestinatario, 
                    $comunaDestinatario, $telefonoDestinatario, $codigoServicio,
                    $numeroTotalPiezas, $kilos, $volumen, $numeroReferencia, 
                    $observaciones, $observaciones2, $emailDestinatario, 
                    $formatoEtiqueta, $valorDeclarado, $pagoSeguro, $tipoMercancia);
                
                srv_envio::ActualizarEnvio_InfoAdmision($order_id, $order_id, $numeroReferencia, $numeroEnvio, $formatoEtiqueta);
    
                $uploadDir   = wp_upload_dir();
                $baseUrl = $uploadDir['baseurl'];
                $baseUrl .= '/correoschile_etiquetas/';
                $extension = 'pdf';
                if($formatoEtiqueta === 'ZPL') {
                    $extension = 'txt';
                }

                $html = '<a href="' . $baseUrl . $numeroEnvio . '.' . $extension . '" target="blank"><strong>' . $numeroEnvio . '</strong></a>';
                header('http/1.1 200 OK');
                header('Content-Type: application/json; charset=UTF8');
                echo json_encode(array('html' => $html, 'jsonData' => $numeroEnvio));
                exit();
            }
            catch (Exception $e) {
                if($tieneEnvio === true) {
                    srv_envio::ActualizarEnvio_InfoError($order_id, $e->getMessage());
                }
                header('http/1.1 500 Internal Server Error');
                header('Content-Type: application/json; charset=UTF8');
                die(json_encode(array('message' => $e->getMessage(), 
                                      'trace' => $e->getTraceAsString())));
            }
        } 
        else 
        {
            header('http/1.1 500 Internal Server Error');
            header('Content-Type: application/json; charset=UTF8');
            die(json_encode(array('message' => 'Este método solo puede ser invocado con POST')));
        }
    }

    public static function ValidarGeneracionEtiqueta($codigoAdmision, $nombreDestinatario, 
        $direccionDestinatario, $comunaDestinatario, $telefonoDestinatario, $codigoServicio, 
        $numeroTotalPiezas, $kilos, $volumen, $numeroReferencia, $observaciones, 
        $observaciones2, $emailDestinatario, $formatoEtiqueta, $valorDeclarado, $esSucursal,
        $pagoSeguro, $tipoMercancia) {
        $mensaje = '';
        try {
            $remitenteNormalizado = srv_normalizacion::NormalizarDireccion(get_option('correoschile_direccion',''), get_option('correoschile_comuna',''));
        }
        catch (Exception $ex) {
            $mensaje.= ($mensaje !== '' ? '<br>':'') . 'La dirección de remitente no se encuentra normalizada.';
        }
        if($esSucursal === 'N') {
            try {
                $destinatarioNormalizado = srv_normalizacion::NormalizarDireccion($direccionDestinatario, $comunaDestinatario);
            }
            catch (Exception $ex) {
                $mensaje.= ($mensaje !== '' ? '<br>':'') . 'La dirección del destinatario no se encuentra normalizada.';
            }
        }    
        if($mensaje !== '') {
            throw new Exception($mensaje);
        }
    }

    public static function GenerarEtiqueta($codigoAdmision, $nombreDestinatario, 
    $direccionDestinatario, $comunaDestinatario, $telefonoDestinatario, $codigoServicio, 
    $numeroTotalPiezas, $kilos, $volumen, $numeroReferencia, $observaciones, 
    $observaciones2, $emailDestinatario, $formatoEtiqueta, $valorDeclarado,
    $pagoSeguro, $tipoMercancia) {
        $data = array(
            'Usuario' => get_option('correoschile_usuario_api',''), //string
            'Contrasena' => get_option('correoschile_contrasena_api',''), //string
            'admisionTO' => array(
                'CodigoAdmision' => $codigoAdmision, //string
                'ClienteRemitente' => get_option('correoschile_codigoSAP',''), //string
                'CentroRemitente' => '', //string
                'NombreRemitente' => get_option('correoschile_nombre_tienda',''), //string
                'DireccionRemitente' => utilidades::LimpiarCaracteres(get_option('correoschile_direccion','')), //string
                'PaisRemitente' => '056', //string
                'CodigoPostalRemitente' => get_option('correoschile_codigoPostal',''), //string
                'ComunaRemitente' => get_option('correoschile_comuna',''), //string
                'RutRemitente' => get_option('correoschile_rut',''), //string
                'PersonaContactoRemitente' => get_option('correoschile_nombre',''), //string
                'TelefonoContactoRemitente' => utilidades::SoloNumeros(get_option('correoschile_telefono','')), //string
                'ClienteDestinatario' => '', //string
                'CentroDestinatario' => '', //string
                'NombreDestinatario' => $nombreDestinatario, //string
                'DireccionDestinatario' => utilidades::LimpiarCaracteres($direccionDestinatario), //string
                'PaisDestinatario' => '056', //string
                'CodigoPostalDestinatario' => '', //string
                'ComunaDestinatario' => utilidades::QuitarAcentos($comunaDestinatario), //string
                'RutDestinatario' => '', //string
                'PersonaContactoDestinatario' => $nombreDestinatario, //string
                'TelefonoContactoDestinatario' => utilidades::SoloNumeros($telefonoDestinatario), //string
                'CodigoServicio' => $codigoServicio, //string
                'NumeroTotalPiezas' => $numeroTotalPiezas, //int
                'Kilos' => $kilos, //double
                'Volumen' => $volumen, //double
                'NumeroReferencia' => $numeroReferencia, //string
                'ImporteReembolso' => 0, //long
                'ImporteValorDeclarado' => $valorDeclarado, //long
                'TipoPortes' => 'P', //string
                'Observaciones' => utilidades::IsNullOrEmptyString($observaciones) ? '' : trim($observaciones), //string
                'Observaciones2' => utilidades::IsNullOrEmptyString($observaciones2) ? '' : trim($observaciones2), //string
                'EmailDestino' => utilidades::IsNullOrEmptyString($emailDestinatario) ? '' : trim($emailDestinatario), //string
                'TipoMercancia' => utilidades::IsNullOrEmptyString($tipoMercancia) ? '' : trim($tipoMercancia), //string
                'DevolucionConforme' => 'N', //string
                'NumeroDocumentos' => 0, //long
                'PagoSeguro' => $pagoSeguro, //string
                'FormatoEtiqueta' => $formatoEtiqueta //string
            )
        );

        try {
            $jsonData = json_encode($data);
            require 'guzzle/vendor/autoload.php';
            $request = new \GuzzleHttp\Client([ 'headers' => [ 'Content-Type' => 'application/json' ]]);
            $url = get_option('correoschile_ws_etiquetas', '');
            $response = $request->post($url, ['body' => $jsonData]);
            $header = $response->getHeader('Content-Disposition');
            $body = $response->getBody()->getContents();
            $header_array = explode('=', $header[0]);
            $file_name = $header_array[1];
            $numeroEnvio = explode('.', $file_name)[0];

            $upload_dir   = wp_upload_dir();
            $upload_dirname = $upload_dir['basedir'];
            $upload_dirname .= '/correoschile_etiquetas';
            if(!file_exists($upload_dirname)) {
                wp_mkdir_p($upload_dirname);
            }
            $fullFileName = $upload_dirname . '/' . $file_name;
            $total_bytes = file_put_contents($fullFileName, $body);
            $full_file_name = $fullFileName;
            return $numeroEnvio;
        }
        catch (GuzzleHttp\Exception\RequestException $e) {
            $contenido = $e->getResponse()->getBody();
            $contenidoJson = json_decode($contenido);
            throw new Exception($contenidoJson->Message);
        }
        catch(Exception $e) {
            throw new Exception('Ocurrio un error intentando generar la etiqueta ' . $order_id);
        }
    }
}