<?php

require_once 'srv_envio.php';

class srv_manifiesto {
    public static function AjaxGenerarManifiesto() {
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            try {
                $order_ids = filter_input(INPUT_POST, 'order_ids');
                if(is_null($order_ids))
                    throw new Exception('No se proporciono la variable "order_ids"');
                
                $ids = explode(',', $order_ids);
                $envios = array();
                $n = 1;
                foreach($ids as $id) {
                    if($id === '') {
                        continue;
                    }
                    $order = wc_get_order(intval($id));
                    if(is_bool($order)) {
                        continue;
                    }
                    $envio = srv_envio::ObtenerEnvio($id);
                    if(is_null($envio)) {
                        continue;
                    }
                    if(is_null($envio->numeroEnvio)) {
                        continue;
                    }
                    array_push($envios, array(
                        'n' => $n, 
                        'producto' => $envio->servicio, 
                        'destinatario' => $order->get_shipping_first_name() . ' ' . $order->get_shipping_last_name(), 
                        'comuna' => $order->get_shipping_city(), 
                        'referencia' => $envio->numeroReferencia, 
                        'seguimiento' => $envio->numeroEnvio, 
                        'bultos' => $envio->numeroTotalPiezas));
                    $n++;
                }
                
                /* ESTE CÓDIGO PUEDE SER USADO PARA PRUEBAS
                $envios = array();
                for($i = 1; $i <= 50; $i++)
                {
                    array_push($envios, array('n' => $i, 
                        'producto' => 'PAQUETE EXPRESS DOMICILIO', 
                        'destinatario' => 'Destinatario ' . strval($i) , 
                        'comuna' => 'Comuna ' . strval($i), 
                        'referencia' => 'referencia' . strval($i),
                        'seguimiento' => 'Seguimiento' . strval($i),
                        'bultos' => 1));
                }
                */
                
                if (count($envios) === 0){
                    header('http/1.1 500 Internal Server Error');
                    header('Content-Type: application/json; charset=UTF8');
                    die(json_encode(array('message' => 'No hay envíos para incluir en el manifiesto')));
                } else {
                    $numeroManifiesto = srv_manifiesto::GenerarManifiestoPdf($envios);
                    
                    $uploadDir   = wp_upload_dir();
                    $baseUrl = $uploadDir['baseurl'];
                    $baseUrl .= '/correoschile_manifiestos/';
                    $link = $baseUrl . $numeroManifiesto;
                    header('http/1.1 200 OK');
                    header('Content-Type: application/json; charset=UTF8');
                    echo json_encode(array('link' => $link));
                    exit();
                }
            } 
            catch (Exception $e) {
                header('http/1.1 500 Internal Server Error');
                header('Content-Type: application/json; charset=UTF8');
                die(json_encode(array('message' => $e->getMessage())));
            }
        } else {
            header('http/1.1 500 Internal Server Error');
            header('Content-Type: application/json; charset=UTF8');
            die(json_encode(array('message' => 'Este método solo puede ser invocado con POST')));
        }       
    }
    public static function GenerarManifiestoPdf($envios) {
        try {
            $html = srv_manifiesto::GenerarManifiestoHtml($envios);
            require_once 'dompdf/autoload.inc.php';
            $dompdf = new Dompdf\Dompdf();
            $dompdf->set_paper('Letter', 'portrait');
            $dompdf->loadHtml(utf8_decode($html));
            $dompdf->render();
            $output = $dompdf->output();
            $numeroManifiesto = date('Ymd') . get_option('correoschile_codigoSAP','');
            
            $i = 0;
            $fileName = $numeroManifiesto . '.pdf';
            
            $uploadDir   = wp_upload_dir();
            $baseDir = $uploadDir['basedir'];
            $baseDir .= '/correoschile_manifiestos';
            if(!file_exists($baseDir))
            {
                wp_mkdir_p($baseDir);
            }
            $fullFileName = $baseDir . '/' . $fileName;
            while(file_exists($fullFileName)){
                $i++;
                $fileName = $numeroManifiesto . '_' . strval($i) . '.pdf';
                $fullFileName = $baseDir . '/' . $fileName;
            }
            file_put_contents($fullFileName, $output);
            return $fileName;
        } 
        catch (Exception $ex) {
            throw new Exception('Ocurrio un error intentando generar el PDF con el manifiesto.');
        }
    }
    public static function GenerarManifiestoHtml($envios){
        $fila = 0;
        $filas = 14;
        $pagina = 0;
        $paginas = intval(count($envios) / $filas) + (count($envios) % $filas > 0 ? 1 : 0);
        
        require 'picqer/vendor/autoload.php';
        $barCodeGenerator = new \Picqer\Barcode\BarcodeGeneratorPNG();
        
        $html = srv_manifiesto::GenerarManifiestoInicioHtml();
        foreach($envios as $envio){
            $fila++;
            if($fila === 1){ 
                $html .= srv_manifiesto::GenerarManifiestoInicioPaginaHtml();
                $html .= srv_manifiesto::GenerarManifiestoInicioTablaHtml();
            }
            $html .= srv_manifiesto::GenerarManifiestoFilaTablaHtml($barCodeGenerator, $envio);
            if($fila === $filas){
                $fila = 0;
                $pagina++;
                $html .= srv_manifiesto::GenerarManifiestoFinTablaHtml();
                $html .= srv_manifiesto::GenerarManifiestoPiePaginaHtml($pagina, $paginas);
            }
        }
        if($fila > 0){
            $pagina++;
            $html .= srv_manifiesto::GenerarManifiestoFinTablaHtml();
            $html .= srv_manifiesto::GenerarManifiestoPiePaginaHtml($pagina, $paginas);        
        }
        $html .= srv_manifiesto::GenerarManifiestoFinHtml();
        return $html;
    }
    public static function GenerarManifiestoInicioHtml(){
        return '<html>'
        . '<head>'
        . '<meta charset="UTF-8">'
        . '<meta name="viewport" content="width=device-width, initial-scale=1.0">'
        . '<meta http-equiv="X-UA-Compatible" content="ie=edge">'
        . '<style>'
        . 'html { margin: 10mm 19mm; font-family: "Helvetica", "Arial", sans-serif; }'
        . 'table { border-collapse: collapse; width:100%; font-size:9pt; }'
        . '.tbl_header { border: 1.1pt solid #000; }'
        . '.tbl_body {  width:100%; border-top: 1.1pt solid #000; border-left: 1.1pt solid #000; }'
        . '.tbl_body th { font-size: 8pt; height: 5mm; background-color: #ccc; border-bottom: 1.1pt solid #000; border-right: 1.1pt solid #000; }'
        . '.tbl_body td { background-color: #fff; height: 11mm; font-size: 8pt; border-bottom: 1.1pt solid #000; border-right: 1.1pt solid #000; }'
        . '.tbl_footer { border-top: 1pt solid #000; border-left: 1.1pt solid #000; }'
        . '.tbl_footer th { font-size: 10pt; height: 6mm; background-color: #ccc; border-bottom: 1.1pt solid #000; border-right: 1.1pt solid #000; }'
        . '.tbl_footer td { background-color: #fff; font-size: 10pt; border-bottom: 1.1pt solid #000; border-right: 1.1pt solid #000; }'
        . '</style>'
        . '</head>'
        . '<body>';
    }
    public static function GenerarManifiestoInicioPaginaHtml() {
        $rutCliente = get_option('correoschile_rut','');
        $cuentaCliente = get_option('correoschile_codigoSAP','');
        $numeroManifiesto = date('Ymd') . $cuentaCliente;
        $razonSocial = get_option('correoschile_nombre_tienda','');
        $fechaCreacion = '';
        $comuna = get_option('correoschile_comuna','');
        $usuario = 'WEB SERVICE';
            
        return '<center><h3 style="font-size:13pt; margin: 0;">MANIFIESTO RESUMEN</h4></center>'
        . '<table class="tbl_header">'
        . '<tr>'
        . '<td style="width:44mm;" rowspan="4" align="center">'
      //  . '<img width="150" height="27" src="data:image/png;base64,"/>'
        . '</td>'
        . '<td style="width:26mm; height:6mm;">Rut:</td>'
        . '<td style="width:37mm;">' . srv_manifiesto::HtmlEncoding($rutCliente)  . '</td>'
        . '<td style="width:33mm;">N&uacute;mero Manifiesto:</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($numeroManifiesto) . '</td>'
        . '</tr>'
        . '<tr>'
        . '<td style="height: 6mm;">Raz&oacute;n Social:</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($razonSocial) .  '</td>'
        . '<td>Fecha Creaci&oacute;n:</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($fechaCreacion) .' </td>'
        . '</tr>'
        . '<tr>'
        . '<td style="height: 6mm">Cuenta Cliente:</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($cuentaCliente) . '</td>'
        . '<td>Comuna:</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($comuna) . '</td>'
        . '</tr>'
        . '<tr>'
        . '<td style="height: 6mm;">Usuario:</td>'
        . '<td colspan="3">' . srv_manifiesto::HtmlEncoding($usuario) . '</td>'
        . '</tr>'
        . '</table>';
    }
    public static function GenerarManifiestoInicioTablaHtml(){
        return '<table class="tbl_body" style="margin-top: 2.5mm;">'
        . '<thead>'
        . '<tr>'
        . '<th style="width:11mm;">N&deg;</th>'
        . '<th>PRODUCTO</th>'
        . '<th style="width:30mm; align:left;">DESTINATARIO</th>'
        . '<th style="width:26mm; align:left;">COMUNA</th>'
        . '<th style="width:24mm; align:left;">REFERENCIA</th>'
        . '<th style="width:33mm; align:left;">SEGUIMIENTO</th>'
        . '<th style="width:11mm; align:left;">BULTOS</th>'
        . '</tr>'
        . '</thead>'
        . '<tbody>';
    }
    public static function GenerarManifiestoFilaTablaHtml($barCodeGenerator, $envio){
        return '<tr>'
        . '<td align="right">' . $envio['n'] . '</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($envio['producto']) . '</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($envio['destinatario']) . '</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($envio['comuna']) . '</td>'
        . '<td>' . srv_manifiesto::HtmlEncoding($envio['referencia']) . '</td>'
        . '<td align="center">'
        . '<img src="data:image/png;base64,' . base64_encode($barCodeGenerator->getBarcode($envio['seguimiento'], $barCodeGenerator::TYPE_CODE_128)) . '" width="120">'
        . '<br><small style="">' . $envio['seguimiento'] . '</small>'
        . '</td>'
        . '<td align="right">' . $envio['bultos'] . '</td>'
        . '</tr>';
    }
    public static function GenerarManifiestoFinTablaHtml() {
        return '</tbody></table>';
    }
    public static function GenerarManifiestoPiePaginaHtml($pagina, $paginas) {
        return '<table class="tbl_footer" style="margin-top: 2.5mm;">'
        . '<tr>'
        . '<th style="width:59mm;">CLIENTE</th>'
        . '<th style="width:64mm;">TRANSPORTE</th>'
        . '<th>ADMISI&Oacute;N CORREOS</th>'
        . '</tr><tr><td style="height: 24mm;"></td><td></td><td></td></tr>'
        . '<tr>'
        . '<th>Nombre, RUT, Firma y Fecha</th>'
        . '<th>Nombre, RUT, Firma y Fecha</th>'
        . '<th>Nombre, RUT, Firma y Fecha</th>'
        . '</tr>'
        . '</table>'
        . '<table width="100%" style="page-break-after: always;">'
        . '<tr><td align="right">P&aacute;gina ' . $pagina . '/' . $paginas . '</td></tr>'
        . '</table>';
    }
    public static function GenerarManifiestoFinHtml(){
        return '</body></html>';
    }
    public static function HtmlEncoding($string){
        return mb_convert_encoding($string, 'HTML-ENTITIES');
    }
}
