<?php

require_once 'srv_envio.php';

class srv_retiro {
    public static function AjaxRegistrarRetiro() {
        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            $tieneEnvio = false;
            try {
                $orders_ids = filter_input(INPUT_POST, 'order_ids');
                $fechaRetiro = filter_input(INPUT_POST, 'fecha');
                $horaDesde = filter_input(INPUT_POST, 'horaDesde');
                $horaHasta = filter_input(INPUT_POST, 'horaHasta');
    
                if(!isset($orders_ids)) throw new Exception('No se proporciono la variable "order_id"');
                if(!isset($fechaRetiro)) throw new Exception('No se proporciono la variable "fecha"');
                if(!isset($horaDesde)) throw new Exception('No se proporciono la variable "horaDesde"');
                if(!isset($horaHasta)) throw new Exception('No se proporciono la variable "horaHasta"');
                if(intval($horaDesde) >= intval($horaHasta)) {
                    throw new Exception('Horario "hasta" no puede ser menor al horario "desde"');
                }
                $horaDesde = substr('00000' . $horaDesde . ':00', -5);
                $horaHasta = substr('00000' . $horaHasta . ':00', -5);

                $orders = explode(',',$orders_ids);
                foreach($orders as $order_id) {
                    $order = wc_get_order(intval($order_id));
                    if(is_bool($order)) {
                        throw new Exception('No se encontró la información del pedido ' . $order_id);
                    }
                    $envio = srv_envio::ObtenerEnvio($order_id);
                    if(is_null($envio)) { 
                        throw new Exception('No se encontró la información del envío ' . $order_id);
                    }
                    $tieneEnvio = true;
                    $codigoAdmision = $envio->codigoAdmision;
                    $codigoRetiro = $envio->codigoRetiro;
                    if(is_null($codigoAdmision)) {
                        throw new Exception('El pedido '. $order_id .' aún no cuenta con etiqueta ');
                    }
                    if($reAgendar !== 'S') {
                        if(utilidades::IsNullOrEmptyString($codigoRetiro) === false) {
                            throw new Exception('El pedido '. $order_id .' ya cuenta son una solicitud de retiro, intente nuevamente ');
                        }
                    }
                }
                
                $hoy = getdate();
                $codigoAdmision = str_replace(' ','', trim(get_option('correoschile_nombre_tienda','')) . '_' .$hoy['hours'] . '' . $hoy['minutes'] . '' . $hoy['seconds']);
    
                $fecha = explode('-', $fechaRetiro);
                $fecha = $fecha[2] . '-' . $fecha[1] . '-' . $fecha[0];
               
                $result = srv_retiro::RegistrarRetiro($codigoAdmision, $fecha, $horaDesde, $horaHasta);
                foreach($orders as $order_id) {
                    srv_envio::ActualizarEnvio_InfoRetiro($order_id, strval($result->CodigoRetiro), $fechaRetiro, $horaDesde, $horaHasta);
                }            
                $retiro = $fechaRetiro . ' ' . $horaDesde . '-' . $horaHasta;
                header('http/1.1 200 OK');
                header('Content-Type: application/json; charset=UTF8');
                echo json_encode(array('retiro' => $retiro, 'codRetiro' => strval($result->CodigoRetiro), 'ordenes' => $orders));
                exit();
            }
            catch(Exception $e)
            {
                if($tieneEnvio){
                    srv_envio::ActualizarEnvio_InfoError($order_id, $e->getMessage());
                }
                header('http/1.1 500 Internal Server Error');
                header('Content-Type: application/json; charset=UTF8');
                die(json_encode(array('message' => $e->getMessage() . '|' . $horaDesde . '|' . $horaHasta)));
            }
        } 
        else 
        {
            header('http/1.1 500 Internal Server Error');
            header('Content-Type: application/json; charset=UTF8');
            die(json_encode(array('message' => 'Este método solo puede ser invocado con POST')));
        }
    }

    public static function RegistrarRetiro($codigoAdmision, $fecha, $horaDesde, $horaHasta) {
        $wsdl = trim(get_option('correoschile_ws_retiro', ''));
        if($wsdl !== '' && substr_compare(strtolower($wsdl), '?wsdl', -strlen('?wsdl')) < 0) {
            $wsdl .= '?wsdl';
        }
        try {
            $client = new SoapClient($wsdl, array('trace' => 1, 'exception' => 0));
            $result = $client->__soapCall('registrarRetiro', array(
                'registrarRetiro' => array(
                    'retiroTO' => array(
                        'ExtensionData' => '',
                        'CodigoAdmision' => $codigoAdmision,
                        'ClienteRemitente' => get_option('correoschile_codigoSAP', ''),
                        'CentroRemitente' => '',
                        'NombreRemitente' => get_option('correoschile_nombre', ''),
                        'DireccionRemitente' => get_option('correoschile_direccion', ''),
                        'PaisRemitente' => '056',
                        'CodigoPostalRemitente' => '',
                        'ComunaRemitente' => get_option('correoschile_comuna', ''),
                        'RutRemitente' => '',
                        'PersonaContactoRemitente' => get_option('correoschile_nombre', ''),
                        'TelefonoContactoRemitente' => get_option('correoschile_telefono', ''),
                        'FechaRetiro' => $fecha,
                        'HoraDesde' => $horaDesde,
                        'HoraHasta' => $horaHasta,
                    ),
                    'usuario' => trim(get_option('correoschile_usuario_api', '')), 
                    'contrasena' => trim(get_option('correoschile_contrasena_api', ''))
                )), null, null);
            return $result->registrarRetiroResult;
        }
        catch (SoapFault $e) {
            throw new Exception($e->getMessage(), 0, $e);
        }
        catch (Exception $e) {
            throw new Exception('Ocurrio un error intentando registrar el Retiro.', 0, $e);
        }
    }
}